
------------------------------------------------------------------------
--
-- ShieldFrames
--
-- Copyright (C) 2010  Philipp Schmidt
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to:
--
-- Free Software Foundation, Inc.,
-- 51 Franklin Street, Fifth Floor,
-- Boston, MA  02110-1301, USA.
--
--
------------------------------------------------------------------------


local ShieldFrames = _G.ShieldFrames;

local AceDialog = nil;
local AceRegistry = nil;

local options;


-- #################################################################################################


local function getGeneralOption(info)
	return ShieldFrames.db.profile[info.arg];
end

local function setGeneralOption(info, value)
	ShieldFrames.db.profile[info.arg] = value;
end

local function getAnchorOption(info)	
	return ShieldFrames.db.profile.anchors[info[2]][info.arg];
end

local function setAnchorOption(info, value)
	local anchorName = info[2];	
	
	ShieldFrames.db.profile.anchors[anchorName][info.arg] = value;
	ShieldFrames.Anchors[anchorName]:ApplyProfile();	
end

local function setNewAnchor(info, value)
	
end


-- #################################################################################################


function options_AnchorProto(name)
	return {
		type = "group",
		name = name,
		get = getAnchorOption,
		set = setAnchorOption,
		
		args = {
			general = {
				type = "group",
				name = "General",
				inline = true,
				order = 1,
				
				args = {
					type = {
						type = "select",
						name = "Type",
						order = 1,
						arg = "type",
						values = {
							["ungrouped"] = "Ungrouped",
						},
					},		
					growth = {
						type = "select",
						name = "Growth Direction",
						order = 3,
						arg = "growthDirection",
						values = {
							[-1] = "Down",
							[1] = "Up",
						},
						set = function(info, value)
							setAnchorOption(info, value);
							
							ShieldFrames.Anchors[info[2]]:ReanchorEffects();	
						end						
					},
				},
			},			
			frameAnchor = {
				type = "group",
				name = "Frame - Anchor",
				inline = true,
				order = 2,
				
				args = {
					scale = {
						type = "range",
						name = "Scale",
						order = 1,
						min = 0.1, max = 2.0, step = 0.01,
						arg = "scale",
					},
					width = {
						type = "range",
						name = "Width",
						order = 2,
						min = 10, max = 1000, step = 1,
						arg = "width",
					},
					sep1 = {
						type = "description",
						order = 3,
						name = "",
						width = "full",
						hidden = false,
					},
					posX = {
						type = "range",
						name = "X offset from left",
						order = 4,
						min = 0, max = floor(GetScreenWidth()), step = 0.1,
						arg = "posX",
					},
					posY = {
						type = "range",
						name = "Y offset from bottom",
						order = 5,
						min = 0, max = floor(GetScreenHeight()), step = 0.1,
						arg = "posY",
					},
					sep2 = {
						type = "description",
						order = 6,
						name = "",
						width = "full",
						hidden = false,
					},					
					height = {
						type = "range",
						name = "Anchor Height",
						order = 7,
						min = 1, max = 100, step = 1,
						arg = "anchorHeight",
					},	
					alpha = {
						type = "range",
						name = "Anchor Alpha",
						order = 8,
						min = 0.0, max = 1.0, step = 0.05,
						arg = "anchorAlpha",
					},		
					visible = {
						type = "toggle",
						name = "Show Anchor",
						order = 9,
						arg = "anchorVisible",
					},
				},
			},			
			frameEffect = {
				type = "group",
				name = "Frame - Effect",
				inline = true,
				order = 4,
				
				args = {
					effectHeight = {
						type = "range",
						name = "Effect Height",
						order = 1,
						min = 1, max = 100, step = 1,
						arg = "effectHeight",
					},
					durationHeight = {
						type = "range",
						name = "Duration Height",
						order = 2,
						min = 1, max = 100, step = 1,
						arg = "durationHeight",
					},
					sep1 = {
						type = "description",
						order = 3,
						name = "",
						width = "full",
						hidden = false,
					},					
					effectAlpha = {
						type = "range",
						name = "Effect Alpha",
						order = 4,
						min = 0.0, max = 1.0, step = 0.05,
						arg = "effectAlpha",
					},
					effectSpacing = {
						type = "range",
						name = "Effect Spacing",
						order = 5,
						min = 1, max = 20, step = 1,
						arg = "effectSpacing",
						set = function(info, value)
							setAnchorOption(info, value);
							
							ShieldFrames.Anchors[info[2]]:ReanchorEffects();	
						end
					},				
				},
			},
			effectText = {
				type = "group",
				name = "Effect Text",
				inline = true,
				order = 5,
				
				args = {
					sep2 = {
						type = "description",
						order = 1,
						name = "Here you can customize the data displayed in the effect bar itself. The | separates the left and right aligned part of the text. You can use the following symbols:\nsource name %s, dest name %d, spell name %n, current absorb amount %a, maximum absorb amout %m, current percentage %p",
						width = "full",
						hidden = false,
					},					
					format = {
						type = "input",
						name = "Text Format",
						order = 2,
						arg = "textFormat",
					},					
					height = {
						type = "range",
						name = "Text Height",
						order = 3,
						min = 1, max = 50, step = 1,
						arg = "textHeight",
					},
			
				},
			},
			filter = {
				type = "group",
				name = "Filter",
				inline = true,
				order = 6,
				
				args = {
					appliedByMe = {
						type = "toggle",
						name = "Show only my effects",
						order = 1,
						arg = "filterAppliedByMe",
					},
					appliedOnMe = {
						type = "toggle",
						name = "Show only effects on me",
						order = 2,
						arg = "filterAppliedOnMe",
					},
				},
			},
		},
	};
end

local options = {
	type = "group",
	get = getGeneralOption,
	set = setGeneralOption,
	
	args = {
		general = {
			type = "group",
			name = "General Options",
			cmdInline = true,
			order = 1,
			
			args = {	
				enabled = {
					type = "toggle",
					order = 1,
					name = "Enabled",
					desc = "Enables/Disables this addon",
					get = function() return ShieldFrames:IsEnabled() end,
					set = function()
						if(ShieldFrames:IsEnabled()) then
							ShieldFrames:Disable();
						else
							ShieldFrames:Enable()
						end
					end,
				},
				locked = {
					type = "toggle",
					order = 2,
					name = "Lock Frames",
					desc = "Locks/Unlocks the frame anchors for repositioning",
					get = function() return ShieldFrames.isLocked end,
					set = function()
						if(ShieldFrames.isLocked) then
							ShieldFrames:UnlockAnchors();
						else
							ShieldFrames:LockAnchors();
						end
					end,
				},
			},
		},
		
		anchors = {
			type = "group",
			name = "Frame Anchors",
			order = 2,
			
			args = {
				__add = {
					type = "group",
					inline = true,
					name = "Add new frame anchor",
					
					args = {
						input = {
							type = "input",
							name = "Anchor name",
							validate = function(info, value)
								if(value == "__add") then
									return "This anchor name is reserved for internal use";
								end
							
								if(ShieldFrames.Anchors[value]) then
									return "There is already a frame anchor with that name";
								end
								
								return true;
							end,
							set = function(info, value)
								ShieldFrames:CreateFrameAnchor(value);
								
								info.options.args.anchors.args[value] = options_AnchorProto(value	);
								
								AceRegistry:NotifyChange("ShieldFrames");
							end,
						},
					},
				},
			},
		}
	},
};


-- #################################################################################################
	

local function LoadAnchors()
	local anchors = ShieldFrames.db.profile.anchors;
	local anchorOptions = options.args.anchors.args;
	
	for name, anchor in pairs(anchors) do
		anchorOptions[name] = options_AnchorProto(name);
	end
end

function ShieldFrames:ShowOptions()
	if(not AceDialog) then
		AceDialog = LibStub("AceConfigDialog-3.0");
		AceRegistry = LibStub("AceConfigRegistry-3.0");
		
		LoadAnchors();
		
		LibStub("AceConfig-3.0"):RegisterOptionsTable("ShieldFrames", options);
		AceDialog:SetDefaultSize("ShieldFrames", 835, 525);
	end
	
	AceDialog:Open("ShieldFrames");
end
