
------------------------------------------------------------------------
--
-- ShieldFrames
--
-- Copyright (C) 2010  Philipp Schmidt
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to:
--
-- Free Software Foundation, Inc.,
-- 51 Franklin Street, Fifth Floor,
-- Boston, MA  02110-1301, USA.
--
--
------------------------------------------------------------------------


ShieldFrames = LibStub("AceAddon-3.0"):NewAddon("ShieldFrames", "AceEvent-3.0");

local ShieldFrames = _G.ShieldFrames;
local AbsorbsMonitor = _G.AbsorbsMonitor;

if(not AbsorbsMonitor) then return; end

local FrameAnchors = {};
ShieldFrames.Anchors = FrameAnchors;
ShieldFrames.isLocked = true;


-- #################################################################################################


local defaults = {
	profile = {
		anchors = {},
	},
};


-- #################################################################################################


local function OnAbsorbEvent(event, ...)
	for name, anchor in pairs(FrameAnchors) do
		anchor[event](anchor, ...);
	end
end

function ShieldFrames:CreateFrameAnchor(name)
	local anchor = self.AnchorProto:New(name);	
	self.db.profile.anchors[name] = anchor.profile;
	
	self.Anchors[name] = anchor;
	
	anchor:Show();
end

function ShieldFrames:LockAnchors()
	for name, anchor in pairs(FrameAnchors) do
		anchor:Lock();
	end
	
	self.isLocked = true;
end

function ShieldFrames:UnlockAnchors()
	for name, anchor in pairs(FrameAnchors) do
		anchor:Unlock();
	end
	
	self.isLocked = false;
end

function ShieldFrames:OnInitialize()
	defaults.profile.anchors.Default = self.AnchorProto.defaults();

	self.db = LibStub:GetLibrary("AceDB-3.0"):New("ShieldFramesDB", defaults, true)
	
	--[[
	self.db = {
		profile = {
			anchors = {
				["Default"] = {
					name = "Default",
					type = "ungrouped",
					posX = 600,
					posY = 600,
					scale = 1.0,
					width = 200,
					growthDirection = -1,
					anchorHeight = 20,
					effectHeight = 16,
					durationHeight = 3,
					anchorAlpha = 1.0,
					effectAlpha = 1.0,
					effectSpacing = 3,
					textHeight = 8,
					textFormat = "%d - %n|%a (%p%)",
					anchorVisible = false,					
					filterAppliedByMe = false,
					filterAppliedOnMe = false,
				},
			},
		},
	};]]
	
	for name, anchor in pairs(self.db.profile.anchors) do
		FrameAnchors[name] = self.AnchorProto:New(name, anchor);
	end
end

function ShieldFrames:OnEnable()
	self.AnchorProto.Enable();

	self:RegisterMessage("Absorbs_EffectApplied", OnAbsorbEvent);
	self:RegisterMessage("Absorbs_EffectUpdated", OnAbsorbEvent);
	self:RegisterMessage("Absorbs_EffectRemoved", OnAbsorbEvent);
	
	self:RegisterMessage("Absorbs_AreaCreated", OnAbsorbEvent);
	self:RegisterMessage("Absorbs_AreaUpdated", OnAbsorbEvent);
	self:RegisterMessage("Absorbs_AreaCleared", OnAbsorbEvent);
	
	self:RegisterMessage("Absorbs_UnitUpdated", OnAbsorbEvent);
	self:RegisterMessage("Absorbs_UnitCleared", OnAbsorbEvent);
end

function ShieldFrames:OnDisable()
	for name, anchor in pairs(FrameAnchors) do
		anchor:Hide();
	end
	
	self:UnregisterAllMessages();
end

function ShieldFrames:Test()
	OnAbsorbEvent("Absorbs_EffectApplied", "0x123", "Aegyan", "0x456", "Ceylara", 64205, 2500, 1.0, 8);
	OnAbsorbEvent("Absorbs_EffectApplied", "0x123", "Aegyan", "0x456", "Ceylara", 48066, 2500, 1.0, 30);
end

function ShieldFrames:Test2()
	OnAbsorbEvent("Absorbs_EffectUpdated", "0x456", 48066, 3000, 9);
end

function ShieldFrames:Test3()
	OnAbsorbEvent("Absorbs_EffectRemoved", "0x456", 64205);
end

function ShieldFrames:Test4()
	OnAbsorbEvent("Absorbs_UnitCleared", "0x456");
end
